SystemOrganization addCategory: #'GT-SUnitDebugger'!

!Object methodsFor: '*GT-SUnitDebugger' stamp: 'AndreiChis 12/28/2015 20:22'!
gtDebuggerSUnitPrint
	"I return a textual representation of the object that is used by the SUnit debugger to compare objects using a textual diff."
	
	^ self printString! !

GTDebuggerContextPredicate subclass: #GTSUnitActivationPredicate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-SUnitDebugger'!

!GTSUnitActivationPredicate commentStamp: 'AndreiChis 12/30/2015 10:47' prior: 0!
I decide when the SUnit debugger should be active.!

!GTSUnitActivationPredicate methodsFor: 'testing' stamp: 'AndreiChis 1/20/2016 22:08'!
matchContext: aContext
	
	^ aContext methodClass isTestCase! !

!Dictionary methodsFor: '*GT-SUnitDebugger' stamp: 'AndreiChis 12/28/2015 20:22'!
gtDebuggerSUnitPrint

	^ String streamContents: [ :aStream | 
		self keysSortedSafely
				do: [:key | aStream print: key;
						 space;
						 nextPutAll: '->';	
						 space;			
						 print: (self at: key);
						 cr]].! !

!String methodsFor: '*GT-SUnitDebugger' stamp: 'AndreiChis 12/28/2015 20:22'!
gtDebuggerSUnitPrint

	^ self printString! !

!DateAndTime methodsFor: '*GT-SUnitDebugger' stamp: 'AndreiChis 12/28/2015 20:22'!
gtDebuggerSUnitPrint

	^ String streamContents: [:s | self printSeparateDateAndTimeOn: s]! !

DebugSession subclass: #GTSUnitDebugSession
	instanceVariableNames: 'assertionContext expectedResult actualResult testContext assertionCallerContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-SUnitDebugger'!

!GTSUnitDebugSession commentStamp: 'AndreiChis 12/30/2015 10:44' prior: 0!
I am a debugging session for the SUnit debugger. 
I extract from the execution stack various data needed by the user interface of the SUnit debugger, like the test object and method.

Public API and Key Messages

- process:context: does the initialization of the session
- data is provided using the methods in the accessing protocol

    Instance Variables
	actualResult:		<Object>
	assertionContext:		<Object>
	expectedResult:		<Object>
	testObject:		<Object>!

!GTSUnitDebugSession class methodsFor: 'accessing' stamp: 'AndreiChis 12/4/2013 12:24'!
activationPredicate

	^  GTSUnitActivationPredicate new! !

!GTSUnitDebugSession methodsFor: 'accessing' stamp: 'AndreiChis 1/20/2016 16:01'!
actualResult
	
	^ actualResult! !

!GTSUnitDebugSession methodsFor: 'accessing' stamp: 'AndreiChis 1/20/2016 23:09'!
assertionCallerContext

	^ assertionCallerContext! !

!GTSUnitDebugSession methodsFor: 'accessing' stamp: 'AndreiChis 1/20/2016 16:01'!
expectedResult
	
	^ expectedResult ! !

!GTSUnitDebugSession methodsFor: 'initialization' stamp: 'AndreiChis 1/20/2016 23:06'!
initializeAssestionContextFrom: aContext
	| frameworkTestClasses |
	
	frameworkTestClasses := {TestCase. TestCase superclass}.
	
	assertionContext := aContext findContextSuchThat: [ :anotherContext |
		anotherContext sender isNil or: [ 
			 (frameworkTestClasses anySatisfy: [ :aTestClass |
				 aTestClass includesSelector: anotherContext sender method selector]) not ] ].
	assertionContext 
		ifNil: [ assertionContext := aContext ]
		ifNotNil: [ assertionCallerContext := assertionContext sender].
	assertionCallerContext ifNil: [ assertionCallerContext := assertionContext ]
	
		! !

!GTSUnitDebugSession methodsFor: 'initialization' stamp: 'AndreiChis 1/20/2016 22:34'!
initializeTestContextFrom: aContext

	testContext := aContext findContextSuchThat: [ :anotherContext |
		anotherContext method isTestMethod ].
	testContext ifNil: [ testContext := aContext ]! !

!GTSUnitDebugSession methodsFor: 'testing' stamp: 'AndreiChis 12/17/2013 11:12'!
isActive

	^ self isActive: self testContext! !

!GTSUnitDebugSession methodsFor: 'testing' stamp: 'AndreiChis 12/17/2013 11:11'!
isActive: aContext

	^ aContext 
		ifNil: [ false ] 
		ifNotNil: [ 
			aContext isDead not and: [
				aContext ~= self process suspendedContext or: [aContext willReturn not ] ] ]! !

!GTSUnitDebugSession methodsFor: 'initialization' stamp: 'AndreiChis 1/20/2016 23:18'!
process: aProcess context: aContext

	super process: aProcess context: aContext.

	assertionContext ifNil: [
		self initializeAssestionContextFrom: aContext.
		self initializeTestContextFrom: assertionContext.
		self supportsDiff ifTrue: [ 
			actualResult := assertionContext tempAt: 1.
			expectedResult := assertionContext tempAt: 2 ]  ]! !

!GTSUnitDebugSession methodsFor: 'accessing' stamp: 'AndreiChis 12/29/2015 17:12'!
setUpMethod

	^ self testObject class lookupSelector: #setUp! !

!GTSUnitDebugSession methodsFor: 'testing' stamp: 'AndreiChis 1/20/2016 16:50'!
supportsDiff

	^ assertionContext selector = #assert:equals:! !

!GTSUnitDebugSession methodsFor: 'accessing' stamp: 'AndreiChis 12/29/2015 17:13'!
tearDownMethod

	^ self testObject class lookupSelector: #tearDown! !

!GTSUnitDebugSession methodsFor: 'accessing' stamp: 'AndreiChis 1/20/2016 22:53'!
testContext

	^ testContext! !

!GTSUnitDebugSession methodsFor: 'accessing' stamp: 'AndreiChis 1/20/2016 22:53'!
testObject

	^ testContext receiver! !

DebugAction subclass: #GTSUnitDebuggerJumpToTestAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-SUnitDebugger'!

!GTSUnitDebuggerJumpToTestAction class methodsFor: 'as yet unclassified' stamp: 'AndreiChis 1/20/2016 23:19'!
gtDebuggerActionFor: aDebugger
	<gtStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)! !

!GTSUnitDebuggerJumpToTestAction methodsFor: 'testing' stamp: 'AndreiChis 1/20/2016 23:19'!
appliesToDebugger: aDebugger

	^ aDebugger session class = GTSUnitDebugSession and: [ 
		aDebugger session testContext ~= aDebugger session assertionCallerContext ]! !

!GTSUnitDebuggerJumpToTestAction methodsFor: 'accessing' stamp: 'AndreiChis 1/20/2016 23:13'!
defaultLabel

	^ 'Jump to test method'! !

!GTSUnitDebuggerJumpToTestAction methodsFor: 'accessing' stamp: 'AndreiChis 1/20/2016 23:14'!
defaultOrder

	^ 10! !

!GTSUnitDebuggerJumpToTestAction methodsFor: 'actions' stamp: 'AndreiChis 1/20/2016 23:23'!
executeAction
	self debugger stackPresentation selection: self debugger session testContext! !

!GTSUnitDebuggerJumpToTestAction methodsFor: 'accessing' stamp: 'AndreiChis 1/20/2016 23:14'!
id

	^ GTSUnitDebuggerJumpToTestAction name! !

!GTSUnitDebuggerJumpToTestAction methodsFor: 'as yet unclassified' stamp: 'AndreiChis 1/20/2016 23:23'!
initialize
	super initialize.
	self needsUpdate: false! !

GTGenericStackDebugger subclass: #GTSUnitDebugger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-SUnitDebugger'!

!GTSUnitDebugger commentStamp: 'AndreiChis 12/30/2015 10:48' prior: 0!
I am a custom debugger for SUnit.
I embed a textual diff when an #assert:equals: assertion fails and provide access to the setUp/tearDown methods of a test.
!

!GTSUnitDebugger class methodsFor: 'testing' stamp: 'AndreiChis 11/27/2015 18:55'!
availableAutomatically
	"Should this debugger be taken into account when looking for a debugger to handle an exception."
	
	^ true! !

!GTSUnitDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 1/10/2016 17:43'!
defaultTitle

	^ 'SUnit'! !

!GTSUnitDebugger class methodsFor: 'testing' stamp: 'AndreiChis 12/29/2015 17:00'!
handlesContext: aContext
	| activationPredicate |
	activationPredicate := self sessionClass activationPredicate.
	^ (aContext stackOfSize: self lookupStackSize) 
		anySatisfy: [ :ctx | activationPredicate matches: ctx ] ! !

!GTSUnitDebugger class methodsFor: 'testing' stamp: 'AndreiChis 11/27/2015 18:20'!
lookupStackSize
	^ 10! !

!GTSUnitDebugger class methodsFor: 'tools registry' stamp: 'AndreiChis 12/31/2013 21:01'!
register

	self registerToolsOn: Smalltalk tools.! !

!GTSUnitDebugger class methodsFor: 'tools registry' stamp: 'AndreiChis 12/4/2013 12:14'!
registerToolsOn: registry

	"Add ourselves to registry. See [Smalltalk tools]" 
	registry registerDebugger: self withRank: 70.! !

!GTSUnitDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 12/4/2013 12:14'!
sessionClass

	^ GTSUnitDebugSession! !

!GTSUnitDebugger methodsFor: 'accessing context' stamp: 'AndreiChis 1/20/2016 23:09'!
contextToSelectFrom: aSession
	^ aSession assertionCallerContext! !

!GTSUnitDebugger methodsFor: 'building' stamp: 'AndreiChis 1/10/2016 17:42'!
debuggerStructureIn: browser
	
	browser
		row: #stack;
		row: [ :c | c
			column: #code; 
			column: #test ] span: 2;
		row: #inspector.
		! !

!GTSUnitDebugger methodsFor: 'building' stamp: 'AndreiChis 12/2/2015 14:38'!
debuggerTransmissionsIn: browser

	super debuggerTransmissionsIn: browser.
	
	browser transmit
		from: #stack port: #selection;
		to: #test;
		andShow: [ :aComposite :aContext | 
			self setUpCodeIn: aComposite.
			self tearDownCodeIn: aComposite ]
! !

!GTSUnitDebugger methodsFor: 'building presentations' stamp: 'AndreiChis 1/20/2016 16:51'!
inspectorDiffIn: aComposite

	^ aComposite tabulator 
		with: [ :aTabulator |
			aTabulator when: [ self session supportsDiff].
			aTabulator 
				column: #actual;
				column: #expected.
			aTabulator transmit to: #actual; andShow: [ :anotherComposite |
				anotherComposite  dynamic 
					display: [:anObject |
				 		(GTDebuggerInspector new 
							startOn: self session actualResult) ] ].
			aTabulator transmit to: #expected; andShow: [ :anotherComposite |
				anotherComposite  dynamic 
					display: [:anObject |
				 		(GTDebuggerInspector new 
							startOn: self session expectedResult) ] ] ]! !

!GTSUnitDebugger methodsFor: 'building presentations' stamp: 'AndreiChis 1/20/2016 16:56'!
inspectorIn: aComposite

	(self textualDiffIn: aComposite) title: 'Diff (actual vs. expected)'.
	(self inspectorDiffIn: aComposite) title: 'Objects (actual vs. expected)'.
	super inspectorIn: aComposite.
	self session supportsDiff ifTrue: [ 
		aComposite last title: 'Inspector' ]! !

!GTSUnitDebugger methodsFor: 'building presentations' stamp: 'AndreiChis 12/2/2015 14:36'!
setUpCodeIn: aComposite

	aComposite pharoMethod
		title: 'SetUp';
		format: [ self session setUpMethod sourceCode ];
		smalltalkClass: [  self session testObject class ];
		doItReceiver: [ self session testObject ].! !

!GTSUnitDebugger methodsFor: 'building presentations' stamp: 'AndreiChis 12/2/2015 14:37'!
tearDownCodeIn: aComposite

	aComposite pharoMethod
		title: 'TearDown';
		format: [ self session tearDownMethod sourceCode ];
		smalltalkClass: [  self session testObject class ];
		doItReceiver: [ self session testObject ].! !

!GTSUnitDebugger methodsFor: 'building presentations' stamp: 'AndreiChis 1/20/2016 16:41'!
textualDiffIn: composite

	^ composite diff
			when: [ self session supportsDiff];
			display: [ 
				{self session actualResult gtDebuggerSUnitPrint. self session expectedResult gtDebuggerSUnitPrint} ]! !

!GTSUnitDebugger methodsFor: 'updating' stamp: 'AndreiChis 11/30/2015 23:32'!
updateBrowser

	super updateBrowser.
	"The transmission will select the context of the test case. 
	This should only happen the first time.
	Should find another solution so this method doesn;t have to be overridden"
	((self browser paneNamed: #stack) port: #selection) value: self session interruptedContext! !

!Collection methodsFor: '*GT-SUnitDebugger' stamp: 'AndreiChis 12/28/2015 20:22'!
gtDebuggerSUnitPrint

	^ String streamContents: [:s | self asStringOn: s delimiter: Character cr asString]! !

TestCase subclass: #GTSUnitExampleFailingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-SUnitDebugger'!

!GTSUnitExampleFailingTest commentStamp: 'AndreiChis 12/30/2015 10:39' prior: 0!
I am not a test!! I provide usage examples for the SUnit debugger.!

!GTSUnitExampleFailingTest class methodsFor: 'testing' stamp: 'AndreiChis 1/19/2014 13:32'!
isAbstract
	^ true! !

!GTSUnitExampleFailingTest class methodsFor: 'testing' stamp: 'AndreiChis 11/27/2015 18:14'!
isUnitTest
	^ false! !

!GTSUnitExampleFailingTest methodsFor: 'tests' stamp: 'TudorGirba 1/20/2016 20:54'!
checkWithFailure
	self assert: false description: 'A failure'! !

!GTSUnitExampleFailingTest methodsFor: 'tests' stamp: 'TudorGirba 1/20/2016 20:55'!
intermediaryCheckWithFailure
	40 + 2.
	self checkWithFailure! !

!GTSUnitExampleFailingTest methodsFor: 'tests' stamp: 'AndreiChis 3/26/2014 16:02'!
testArrays
	 self
           assert: #(1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22
23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 
49 50 51 51 51 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 
75 76 77 78 79 80 81 81 81 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99)
           equals: #(1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 
23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 
49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 
75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99)! !

!GTSUnitExampleFailingTest methodsFor: 'tests' stamp: 'AndreiChis 3/26/2014 16:17'!
testDictionary
	 self
		assert: (#(1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22
23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 
49 50 51 51 51 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 
75 76 77 78 79 80 81 81 81 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101) inject: Dictionary new into: [ :dict :nr| dict at: nr put: nr<<5. dict ])
      equals: (#(1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 
23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 
49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 
75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101) inject: Dictionary new into: [ :dict :nr| 
		nr\\40 == 0 
			ifTrue: [ dict at: nr put: nr<<6 ]
			ifFalse: [ dict at: nr put: nr<<5 ].
			dict ])! !

!GTSUnitExampleFailingTest methodsFor: 'tests' stamp: 'TudorGirba 1/20/2016 20:54'!
testFailingAssestionThroughAHelperMethod

	self intermediaryCheckWithFailure! !

!GTSUnitExampleFailingTest methodsFor: 'tests' stamp: 'AndreiChis 3/26/2014 16:01'!
testFailingDate
	|aDateAndTime|

	aDateAndTime :=  DateAndTime localTimeZone: TimeZone default; dosEpoch.
	self 
		assert: (aDateAndTime offset: '0:12:00:00')  
		equals: '1 January 1980 1:00 pm' asDateAndTime! !

!GTSUnitExampleFailingTest methodsFor: 'tests' stamp: 'AndreiChis 1/20/2016 16:15'!
testFailingText

	self assert: 'I like pie  
pizza pie 
apple pie 
rad squared pi 
and other PIes'
	equals: 'I like pie  
pizza pie 
potato pie 
rad squared pi 
and other PIes'! !

!GTSUnitExampleFailingTest methodsFor: 'tests' stamp: 'AndreiChis 12/17/2013 10:50'!
testMultiValuedStreaming
	| headers keysAndValues |
	headers := ZnHeaders new.
	headers at: 'Set-Cookie' put: 'foo=1'.
	headers at: 'Set-Cookie' add: 'bar=1'.
	headers at: 'Content-TYPE' put: 'text/plain'.
	headers at: 'Content-Length' put: '128'.
	keysAndValues := OrderedCollection streamContents: [ : stream |
		headers headersDo: [ :key :value |
			stream nextPut: key; nextPut: value ] ].
	self assert: keysAndValues equals: #('Set-Cookie' 'foo=1' 'Set-Cookie' 'bar=1' 'Content-Length' '128' 'Content-TYPE' 'text/plain') asOrderedCollection 
	
	! !

!GTSUnitExampleFailingTest methodsFor: 'tests' stamp: 'AndreiChis 1/20/2016 16:06'!
testSimpleFailingAssestion

	self assert: false! !
